/*================================================================
               RJESENJE ZADATKA POPO
 
 Najprije je potrebno primjetiti da relaciju 'nadredjena osoba' 
 kako je opisana u zadatku mozemo predstaviti stablom. To je zato
 sto zadana relacija nema ciklusa (jer, kako kaze zadatak, slijed 
 nadredjenih osoba vodi do kralja), iz istog razloga postoji samo 
 jedan korijen, i jos je zadano da svaki cvor ima samo jednog roditelja.
 To sve zajedno je dovoljno za zakljucak o stablu.
 
 Potrebno je uociti da pohlepno rijesenje radi uspjesno 
 ako se drzimo pravila navedenih u zadatku. Dakle, krenemo 
 od listova i obradjujemo neki cvor tek kada smo obradili sve 
 njegove 'podredjene' cvorove. Kada god mozemo uzeti da je cvor
 porezni obveznik, to napravimo s tim da oznacimo njemu nadredjen 
 cvor kao sakupljac poreza.
 
 Ovo funkcionira iz razloga sto:
 - ako neki promatrani cvor ima samo listove za podredjene cvorove, 
   uvijek je bolje uzeti podredjeni cvor kao poreznog obveznika 
   nego promatrani cvor (jer, ako bi uzeli promatrani cvor za poreznog 
   obveznika onda ne bi smjeli uzeti niti jednog podredjenog cvora za
   poreznog obveznika, a to je uvijek losije)
     
     
 Potrebno je samo ucitati podatke, organizirati ih u stablo i pratiti osobne 
 brojeve. U ovom programu je svakoj osobi dodjeljen jos i indeks od 1 do N 
 prema redu ucitavanja tako da bude lakse ostvariti relaciju 'nadredjena osoba'.
 
 To je sve skupa u ovom programu izvedeno poljima koja su sva indeksirana upravo
 spomenutim indeksom:
 - brPodanika - sadrzava osobni broj kralja ili podanika
 - nad - sadrzava indeks nadredjene osobe
 - jeLiSakupljac - oznacava mora li osoba biti sakupljac poreza (jer je neki 
      njen podanik uzet za poreznog obveznika)
 
 I naposlijetku, red 'listovi' na sluzi kako bismo cvorove obradili redom od
 listova prema kraju kako je opisano, dakle kao pretraga u sirinu ali krenuvsi od kraja.
     
     
 Kresimir Cosic
 
*/

#include <iostream>
#include <fstream>
#include <vector>
#include <queue>
#include <string>
#include <algorithm>
#include <assert.h>
using namespace std;

const int maxBr=2*1000*1000;

int main()
{
    istream& ulDat= cin;
    //ifstream ulDat("popoUlaz.txt");            
	//ifstream ulDat("test.0.in");            
    
    int n;
    ulDat >> n;   
    
    vector<int>  nad(maxBr);
    vector<int> brPodanika(maxBr);    
    queue<int> listovi;
    
    for (int i=0;i<n;i++)
		{
		int brOsobe, brNad;
		ulDat >> brOsobe;
		ulDat >> brNad;
		nad[brOsobe]=brNad;
		brPodanika[brNad]++;
		}
		
	for (int i=0;i<maxBr;i++)
		if (nad[i]!=0 && brPodanika[i]==0)
			listovi.push(i);
	
	vector<char> jeLiSakupljac(maxBr);	
	int brObveznika=0;
	
	while (!listovi.empty()){
		int br=listovi.front();
		listovi.pop();
		
		if (br==1) break;		
		int brNad=nad[br];
		
		if (!jeLiSakupljac[br])
			{
			brObveznika++;			
			jeLiSakupljac[brNad]=1;						
			}
		
		if (brPodanika[brNad]>1)
			brPodanika[brNad]--;
		else
			listovi.push(brNad);								
		}
	
	cout << brObveznika << endl;               
}

