/*
* Opis rjesenja:
* -------------------------
* Na pocetku je potrebno primjetiti da se zadatak mogao postaviti i na drugaciji nacin.
* Okupljanje vojnika na bilo kojem mjestu na mapi je ekvivalentno okupljanju kod zastave s obzirom
* da nas zanima samo maksimalna moguca udaljenost.

* U prvom koraku algoritma potrebno je za svako polje na mapi izracunati 
* udaljenost najblizeg vojnika vojske Y (matrica closest).
* Ovaj korak se moze napraviti u slozenosti N*N koristenjem BFS algoritma.
*
* Zatim se koristenjem postupka binarnog pretrazivanja nad trazenim rjesenjem (binary-search) 
* zadatak svodi na problem odlucivanja da li je za neku fiksnu maksimalnu udaljenost do najblizeg vojnika vojske Y
* moguce da svi vojnici vojske X dodu do zastave. To se moze ostvariti BFS algoritmom u slozenosti N * N.
* Stoga ukupna slozenost postupka iznosi: O(N*N + N*N*logN)

* II. rjesenje: drugi dio zadatka takoder je moguce rijesiti primjenom Dijsktrinog algoritma bez potrebe za
* binary search postupkom. Ukupna slozenost je u tom slucaju nepromjenjena.

	Igor Bonai
*/

#include <cstdio>
#include <cassert>
#include <iostream>
#include <queue>
#include <climits>
using namespace std;

#define N 1000
#define INF INT_MAX / 2

char polje[N][N];
int closest[N][N];
int n, m;
int x_flag, y_flag;

vector < pair<int,int> > X, Y; //pozicije vojnika X i Y na mapi

int move_x[] = {-1, 0, 0, +1};
int move_y[] = {0, -1, 1, 0};
#define MOVES_COUNT (sizeof(move_x) / sizeof(int))

bool ok(int x, int y) {
	if (x < 0 || x >= n) return false;
	if (y < 0 || y >= m) return false;
	return true;
}

void calc_closest_Y() {
	queue < pair<int,int> > q;

	for(int i=0; i<N; i++)
		fill( closest[i], closest[i] + N, INF );

	for(int i=0; i<Y.size(); i++) {
		q.push( make_pair( Y[i].first, Y[i].second) );
		closest[ Y[i].first ][ Y[i].second ] = 0;
	}

	for(; q.size(); q.pop() ) {
		int x = q.front().first;
		int y = q.front().second;
		int d = closest[x][y];

		for(int i=0; i<MOVES_COUNT; i++) {
			int nx = x + move_x[i];
			int ny = y + move_y[i];

			if (!ok(nx, ny)) continue;

			if (closest[nx][ny] > d + 1) {
				closest[nx][ny] = d + 1;
				q.push( make_pair(nx, ny) );
			}
		}
	}	
}

void load() {
	scanf("%d%d", &n, &m);
	for(int i=0; i<n; i++) {
		scanf("%s", polje[i]);	
		for(int j=0; j<m; j++) {
			if (polje[i][j] == 'X')
				X.push_back( make_pair(i,j) );
			if (polje[i][j] == 'Y')
				Y.push_back( make_pair(i,j) );
			if (polje[i][j] == 'F') {
				x_flag = i;
				y_flag = j;
			}
		}
	}
}

bool check(int alowed_dist) {
	vector< vector<bool> > reach( n, vector<bool>(m, false ) );
	queue< pair<int,int> > q;
	
	q.push( make_pair(x_flag, y_flag) );
	reach[ x_flag ][ y_flag ] = true;

	int reached_X = 0;

	//provjeri pocetne
	for(int i=0; i<X.size(); i++) {
		int x = X[i].first;
		int y = X[i].second;
		if ( closest[x][y] < alowed_dist) 
			return false;
	}

	if (closest[ x_flag ][ y_flag ] < alowed_dist)
		return false;

	for(; q.size(); q.pop() ) {
		int x = q.front().first;
		int y = q.front().second;

		for(int i=0; i<MOVES_COUNT; i++) {
			int nx = x + move_x[i];
			int ny = y + move_y[i];

			if (!ok(nx, ny)) continue;
			if (closest[nx][ny] < alowed_dist) continue;

			if (reach[nx][ny] == false) {
				reach[nx][ny] = true;
				q.push( make_pair(nx, ny) );

				if (polje[nx][ny] == 'X')
					reached_X ++;
			}
		}
	}
	
	assert( reached_X <= X.size() );
	return reached_X == X.size();
}

int solve(int start, int end) {
	int middle = (start + end) / 2;

	if (start == end) return start;
	
	if (end - start == 1) {
		if (check(end) == true) return end;
		else {
			return start;
		}
	}
	
	if ( check(middle) == true)
		return solve( middle, end);
	else
		return solve( start, middle-1);
}

int main(void) {
	load();
	calc_closest_Y();
	
	cout << solve( 0, n ) << endl;
	return 0;
}
