/*	
	Dravno natjecanje 2011 - 2. dan
	I.Podskupina
	Zadatak: Svemir

		Opis rjeenja
	------------------------------------
	U zadatku je potrebno uoiti da svaki frizbi, pri ovakvoj igri, uvijek ue u ciklus.

	Da bi se uspjeno rjeio zadatak, potrebno je simulacijom izraunati kada e koji 
	frizbi doi kod kojeg igraa. Simulaciju radimo dok god ne pronaemo oba ciklusa. 
	Tada smo sigurni da su frizbiji doli do svih igraa do kojih mogu doi.

	Treba primjetiti i da se svi igrai ne nalaze nuno u ciklusu, pa ako doe do sudara 
	kod takvih igraa, to emo zapamtiti tokom simulacije.

	Ovakva simulacija nee detektirati sve mogue sudare, ve samo one koji e se dogoditi 
	izvan ciklusa. Meutim za detekciju sudara unutar ciklusa, potrebno je detaljnije razmotriti 
	situaciju.

	Frizbi e se kod osobe x nai u trenucima Tx, pri emu vrijedi:
						
						Tx = t + k*C,

	gdje t oznaava vrijeme kada se frizbi naao prvi put kod osobe x, C oznaava duinu ciklusa, 
	a k je prirodan broj ili nula, koji oznaava broj ciklusa. Na ovaj nain moemo izraunati 
	sve trenutke u kojima se frizbi naao u osobe x.

	Da bi se kod neke osobe dva frizbija nala istovremeno, potrebno je izjednaiti formule za
	oba frizbija. Dobijemo:

						tr + k1*Cr = ty + k2*Cy,

	iz ega dobijemo:
						k1*Cr - k2*Cy = ty-tr,

	pri emu su nam nepoznanice k1 i k2 koji predstavljaju broj ciklusa crvenog i utog frizbija.
	Ova jednadba ima rjeenje samo ako je (ty - tr) % Gcd(Cr, Cy) = 0.

	Ako je ispunjen taj uvjet, tada je mogue da se sudar dogodi kod te osobe.
*/

#include <cstdio>
#include <vector>
using namespace std;

int Abs (int x) { return (x < 0) ? -x : x; }
int Gcd(int a, int b) { return (b == 0) ? a : Gcd(b, a % b); }

struct player
{
	player()
	{
		redFriend = -1;
		yellowFriend = -1;
		redTime = -1;
		yellowTime = -1;
		counted = false;
	}
	int redFriend, yellowFriend;
	int redTime, yellowTime;
	bool counted;
};

int redStart, yellowStart;
int redCycleStartTime, yellowCycleStartTime;
int redCycleLength, yellowCycleLength;
vector<player> players;

int Simulate()
{
	bool isRedCycle = false;
	bool isYellowCycle = false;

	int currentRed = redStart;
	int currentYellow = yellowStart;
	
	int count = 0;
	int time = 0;

	while (!isRedCycle || !isYellowCycle)
	{
		if (currentRed == currentYellow)
		{
			count ++;
			players[currentRed].counted = true;
		}

		if (players[currentRed].redTime == -1) 
		{
			players[currentRed].redTime = time;
		}
		else if (!isRedCycle)
		{
			isRedCycle = true;
			redCycleStartTime = players[currentRed].redTime;
			redCycleLength = time - redCycleStartTime;
		}

		if (players[currentYellow].yellowTime == -1) 
		{
			players[currentYellow].yellowTime = time;
		}
		else if (!isYellowCycle)
		{
			isYellowCycle = true;
			yellowCycleStartTime = players[currentYellow].yellowTime;
			yellowCycleLength = time - yellowCycleStartTime;
		}

		currentRed = players[currentRed].redFriend;
		currentYellow = players[currentYellow].yellowFriend;
		time++;
	}

	return count;
}

int Check (player P)
{
	if (Abs(P.redTime-P.yellowTime) % Gcd(redCycleLength, yellowCycleLength) == 0)
		return 1;

	return 0;
}

int Solve()
{
	int count = 0;

	for (int i=0; i<players.size(); i++)
	{
		if (!players[i].counted && players[i].redTime >= redCycleStartTime && players[i].yellowTime >= yellowCycleStartTime)
			count += Check(players[i]);		
	}

	return count;
}

int main ()
{
	int N;
	int R,Y;
	scanf("%d", &N);
	
	scanf("%d %d", &redStart, &yellowStart);
	--redStart; --yellowStart;

	players.resize(N);

	scanf("%d", &R);

	for (int i=0; i<R; i++)
	{
		int a,b;
		scanf("%d %d", &a, &b);
		--a; --b;

		players[a].redFriend = b;
	}

	scanf("%d", &Y);

	for (int i=0; i<Y; i++)
	{
		int a,b;
		scanf("%d %d", &a, &b);
		--a; --b;

		players[a].yellowFriend = b;
	}

	int solution = Simulate() + Solve();

	printf("%d\n", solution);

	return 0;
}