/*
	Opis rjeenja
--------------------------------------------
U zadatku je bilo potrebno primjetiti da sve sektore moemo
shvatiti kao vorove grafa a uvjet o broju policajaca koji smije
ui na neko polje i nekog smjera kao teinu tog brida. Kada
dodamo jo i uvjet naina na koji se kreu policajci postaje nam
jasno da se radi o usmjerenom teinskom grafu.

Nakon to izgradimo takav graf zadatak se dalje rjeava max-flow algoritmom. 
Svi bridovi su nam kapaciteti kroz koje mogu "tei" policajci :)
Potrebno je napraviti virtualni izvor(source) i spojiti ga kapacitetom 
vrijednosti 1 do svakog polja na kojem se nalazi policajac. 
Ponor(sink) u ovom sluaju e biti polje na kojem se nalazi kriminalac.

*/
#include <iostream>
#include <queue>
#include <vector>
#include <set>
using namespace std;

#define x first
#define y second
#define mp(a,b) make_pair(a,b)

int N, M, K, XK, YK;
int maze[50][50] = { 0 };
vector< pair<int,int> > police;

int capacity[51][51][51][51] = { 0 };
vector< vector <pair<int,int> > > previous;
bool visited[51][51];
queue< pair<int, int> > Q;

void Init()
{
	cin >> N >> M;
	cin >> XK >> YK;

	previous.resize(51);
	for (int i = 0; i < 51; ++i) previous[i].resize(51);

	for (int i = 0; i < N; ++i)
	{
		for (int j = 0; j < M; ++j)
			cin >> maze[i][j];
	}

	cin >> K;
	police.resize(K);
	for (int i = 0; i < K; ++i)
		cin >> police[i].x >> police[i].y;

}

void BuildGraph()
{
	for (int i = 0; i < N; ++i)
		for (int j = 0; j < M; ++j)
		{
			if (i > 0) capacity[i-1][j][i][j] = maze[i][j] / 3; 
			if (j > 0) capacity[i][j-1][i][j] = maze[i][j] / 3;
		}

	for (int i = 0; i < police.size(); ++i)
		capacity[50][50][police[i].x-1][police[i].y-1]++; // (50,50) je source
}

int FindAugmentingPath(pair<int,int> source, pair<int,int> sink) //Finds an aug. path using BFS
{
	// Initialize
	while (!Q.empty()) Q.pop();
	for (int i = 0; i <= N; ++i)
	{
		for( int j = 0; j <= M; ++j)
		{
			previous[i][j] = mp(-1,-1);
			visited[i][j] = false;
		}
	}

	// BFS - traimo augmenting path
	visited[source.x][source.y] = true;
	previous[source.x][source.y] = mp(-1,-1);
	for (Q.push(source); !Q.empty(); Q.pop())
	{
		pair<int,int> node = Q.front();

		if (node == sink) 
			break;

		int nx = node.x;
		int ny = node.y;

		if (nx == 50 && ny == 50) // Poseban slucaj za source
			for (int i = 0; i < N; ++i)
				for (int j = 0; j < M; ++j)
					if (capacity[nx][ny][i][j] > 0 && !visited[i][j])
					{
						Q.push(mp(i, j));
						previous[i][j] = node;
						visited[i][j]  = true;
					}


		if (nx+1 <= N && capacity[nx][ny][nx+1][ny] > 0 && !visited[nx+1][ny])
		{
			Q.push(mp(nx+1, ny));
			previous[nx+1][ny] = node;
			visited[nx+1][ny]  = true;
		}
		if (ny+1 <= M && capacity[nx][ny][nx][ny+1] > 0 && !visited[nx][ny+1])
		{
			Q.push(mp(nx, ny+1));
			previous[nx][ny+1] = node;
			visited[nx][ny+1]  = true;
		}

	}

	// Raunamo minimalni flow kroz cijeli augmenting path
	int minimumCapacity = INT_MAX;
	pair<int,int> node = sink;
	while (previous[node.x][node.y].x != -1 || previous[node.x][node.y].y != -1 )
	{
		pair <int,int> prev = previous[node.x][node.y];
		minimumCapacity = min(minimumCapacity, capacity[prev.x][prev.y][node.x][node.y]);
		node = prev;
	}

	// Auriramo rezidualnu mreu
	pair<int,int> current = sink;
	while (previous[current.x][current.y].x != -1 || previous[current.x][current.y].y != -1)
	{
		pair<int,int> prev = previous[current.x][current.y];
		
		capacity[prev.x][prev.y][current.x][current.y] -= minimumCapacity;
		capacity[current.x][current.y][prev.x][prev.y] += minimumCapacity;
		
		current = prev;
	}

	if (minimumCapacity == INT_MAX)
		return 0;
	else
		return minimumCapacity;
}

int MaximumFlow(pair<int,int> source, pair<int,int> sink)
{
	int maximumFlow = 0;
	while (true)
	{
		int flow = FindAugmentingPath(source, sink);

		if (flow > 0)
			maximumFlow += flow;
		else
			break;
	}
	return maximumFlow;
}

int main()
{
	Init();
	BuildGraph();

	cout << MaximumFlow(mp(50,50), mp(XK - 1, YK - 1)) << endl;

	return 0;
}
            
            
