/*================================================================
               RJESENJE ZADATAKA AMTEL I JURTEK

 Potrebno je najprije primjetiti da krug Z0 zbraja dva bita (dva 
 jednobitna broja, dva jednoznamenkasta binarna broja) na svojim 
 ulazima, a izlaz je dvobitni (dvoznamenkasti) binarni broj. 
 Pritom izlaz i2 sadrzava bit (znamenku) rezultata koji je manje 
 tezine, i iste tezine kao obje ulazne znamenke, a izlaz i1 
 predstavlja bit (znamenku) vece tezine.
 
 Krug Z1 radi slicno samo sto rezultatu jos pribraja 
 broj 1, dakle zbraja dva jednoznamenkasta binarna broja i jos im 
 dodaje broj 1.
 
 Drugi nuzan zakljucak je da moramo uvijek zbrajati znamenke (bitove)
 iste tezine medjusobno.
 
 Rezultat zbrajanja dva broja s N znamenaka uvijek ima N+1 znamenaka.
 
 Na primjeru 1 je vidljiv nacin kako mozemo sastaviti 
 zbrajalo primjenom iskljucivo krugova Z0. Dokaz je induktivan:
 1. Z0 zbraja dva jednoznamenkasta (binarna) broja, tim krugom samim 
    dobijemo zbrajalo jednoznamenkastih (binarnih) brojeva.
 2. Za zbrajanje dva broja s N+1 znamenaka uzmemo zbrajalo koje 
    zbraja N znamenaka, znamenku najvece tezine tog rezultata 
    moramo pribrojiti dvama znamenkama ulaznih brojeva najvece 
    tezine (sve te tri zanmenke su iste tezine), dakle moramo 
    zbrojiti 3 jednobitne znamenke, a takvo sto prikazano 
    je na primjeru 1 (uklonimo iz primjera desni gornji krug Z0).
    
 Pri tome nam je potrebno, za zbrajanje 2 N-znamenkasta broja
  2*N - 1 nivoa (***)
  3*N - 2 krugova
 
   (***) (necemo brojati ulazne i izlazne nivoe u cijelom
   ovom objasnjenju, njih cemo samo pribrojiti na kraju programa 
   za AMTEL ili oduzeti na pocetku programa za JURTEK).
   
   
 Drugi nacin sastavljanja zbrajala prikazan je na primjeru 2.
 U ovom slucaju logika nije induktivna nego rekurzivna, i to 
 ovako: 
 
 - Oznacimo sa Z0[N] zbrajalo koje zbraja dva N-teroznamenkasta 
   binarna broja u NIV(Z0[N]) nivoa, 
 - Jos oznacimo sa Z1[N] zbrajalo koje 
   zbraja dva N-teroznamenkasta binarna broja i jos im dodaje 
   broj 1, i to sve u NIV(Z1[N]) nivoa. 
   
 - Znamo sastaviti Z0[1] (to je Z0) i Z1[1] (to je Z1)
 
 - Ako znamo sastaviti Z1[U], Z0[U], Z0[L], tada znamo 
   sastaviti i Z0[U+L] , a slijedi dugo objasnjenje kako tocno.
   
   Ovaj cijeli postupak moze se vidjeti na primjeru 2, kod 
   kojeg je U=1 i L=1.
   
   To napravomo na nacin da uzmemo jedno zbrajalo Z1[U],
   jedno Z0[U] i jedno Z0[L], i postavimo ih jedan do drugog,
   dakle u MAX( NIV(Z1[U]), NIV(Z0[U]), NIV(Z0[L]) ) nivoa. 
   Ne zaboravite, sastavljamo Z0[U+L], koje zbraja dva broja s
   po U+L znamenaka, pa na zbrajalo Z0[L] dovedemo L znamenaka 
   manje tezine ulaznih brojeva, a preostalih U znamenaka 
   vece tezine ulaznih brojeva dovedemo i na Z0[U] i na Z1[U].
   
   Odmah je jasno da ce zbrajalo Z0[L] dati L ispravnih  
   znamenaka rezultata najmanje tezine, i jos jednu znamenku 
   prijenosa koju treba pribrojiti znamenkama vece tezine.
   
   Zatim primjetimo da, ako je ta znamenka prijenosa jednaka 0, onda
   nam krug Z0[U] daje sve ispravne preostale znamenke rezultata,
   a ako je ta znamenka jednaka 1 onda istu stvar radi Z1[U].
   
   Odmah je jasno da cemo te znamenke rezultata odabrati pomocu
   logickih krugova K (opisanog u tekstu zadatka), kojima cemo 
   svima na ulaz Is postaviti znamenku prijenosa. Za to ce 
   nam trebati U+1 krugova tipa K jer zbrajala Z0[U] i Z1[U] imaju , 
   ocito, svako po U+1 izlaznih znamenaka.
   
   Za sastavljanje zbrajala Z1[U+L] radimo potpuno isto kao za 
   zbrajalo Z0[U+L], samo sto koristimo krugove Z1[U], Z0[U], Z1[L].
   Time je ocito da ce zbrajala Z0[N] i Z1[N] imati istu 
   strukturu, broj logickih krugova i broj nivoa.
   
   Vrijedi:   
   NIV(Z1[N]) = NIV(Z0[N])
   
   Kako logicke krugove K moramo staviti u novi nivo, to ce nam 
   za krug Z0[K+L] trebati 1 + MAX( NIV(Z1[U]), NIV(Z0[U]), NIV(Z0[L]) )
   nivoa.
   
   Broj logickih krugova kruga Z0[N] oznacimo s BR(Z0[N]), a 
   broj logickih krugova kruga Z1[N] oznacimo s BR(Z1[N]).
   Racunamo koliko nam treba logickih krugova za ostvarenje zbrajala
   Vrijedi:      
   BR(Z0[U+L]) = BR(Z0[L]) + BR(Z0[U]) + BR(Z1[U]) + U+1
     (ovaj U+1 na kraju je za logicke krugove K).
   i jos vrijedi: BR(Z1[N]) = BR(Z0[N])
   
     
 - Sada nam jos ostaje primjetiti da ako slazemo krugove upravo 
   opisanim postupkom, onda ce to rezultirati zbrajalom s manjim brojem 
   nivoa i s vecim brojem logickih krugova, poput onog na primjeru 2.
   
   Ako konstruiramo zbrajalo samo primjenom krugova Z0, kao na primjeru 1,
   tada ce nam trebati manje logickih krugova, ali vise nivoa.
   
   Metodom iscrplivanja se mozemo uvjeriti da ne postoji niti jedan bolji 
   nacin da zbrajamo brojeve. Broj primjera koji je potrebno 
   provjeriti da se uvjerimo da nema boljeg nacina je vrlo malen ako 
   postujemo cinjenicu da smijemo medjusobno zbrajati samo znamenke iste 
   tezine (a ako ne postujemo tu cinjenicu ocito je da ne mozemo nista dobiti).
   
 - Oba postupka mozemo objediniti jednom rekurzivnom relacijom koja 
   racuna apsolutni minimum broja logickih krugova za zadani broj
   znamenaka i maksimalni broj nivoa. Za ovu rekurzivnu relaciju se odmah vidi
   da se moze izracunati dinamickim programiranjem, uz koristenje
   dvodimenzionalnog polja za memoizaciju. Stupci tog polja predstavljati 
   ce broj znamenaka, a retci ce predstavljati broj nivoa. Vrijednost
   u pojedinom retku i stupcu predstavlja minimum broja logickih krugova.
   
   Bas taj algoritam implementiran je u programu.
   
   Podatak koji se trazi kao izlaz programa lako je dobiti iz spomenutog 
   polja.
   
   
   Kresimir Cosic

============================================================*/

#define JURTEK
//DEFINE AMTEL

#include <iostream>
#include <fstream>
#include <vector>
#include <queue>
#include <string>
#include <algorithm>
#include <assert.h>
using namespace std;

const int maxBitN=256;
const int maxLevelN=1001;

const int tagUndoable=100*1000*1000;

vector< vector<int> >  din(maxLevelN,vector<int>(maxBitN+1,tagUndoable));

int Min(int a, int b) {  return a<b? a:b; }

int& memMinBrKrug(int levelI, int bitI)
{
	return din[levelI][bitI];
}

int main()
{
    istream& ulDat= cin;
    //ifstream ulDat("zbrajaloUlaz.txt");            
    
    int bitN;
    ulDat >> bitN;   

	#ifdef JURTEK
		int levelN;
		ulDat >> levelN; 
		levelN-=2;        
	#endif
	
	#ifdef AMTEL
		int circuitN;
		ulDat >> circuitN;
	#endif

    
	
	for (int bitI=1; bitI<=bitN; bitI++)		
		for (int levelI=1; levelI<maxLevelN; levelI++)		
			{
			if (levelI==bitI*2-1) {
				//known number of circuits for ripple carry adder (as in example 1)
				memMinBrKrug(levelI,bitI)=3*bitI-2;
				continue;
				}
				
			int minBrKrug=2*1000*1000*1000;
			for (int bitSplit=1;bitSplit<bitI;bitSplit++){
				
				//bits below split on carry
				int bitIOfLowBits=bitI-bitSplit;
				int brKrug=memMinBrKrug(levelI-1,bitIOfLowBits);
				
				//bits above split on carry
				
				//double number of bits - once for carry 0 and once for carry 1
				brKrug+= 2 * memMinBrKrug(levelI-1,bitSplit);
				//add the number of kombinator chips required
				brKrug+= bitSplit+1;
				
				brKrug=Min(brKrug,tagUndoable);

				minBrKrug=Min(minBrKrug,brKrug);
				}
			
			for (int i=1;i<levelI;i++)
				minBrKrug=Min(minBrKrug,memMinBrKrug(i,bitI));
			
			memMinBrKrug(levelI,bitI)=minBrKrug;		
			}
	
	#ifdef JURTEK
		int rez=memMinBrKrug(levelN,bitN);
		if (rez>=tagUndoable)
			cout << -1;
		else
			cout << rez << endl;		
	#endif
	
	#ifdef AMTEL	
		int minLevel;	
		for (minLevel=1; minLevel<maxLevelN; minLevel++)		
			if (memMinBrKrug(minLevel,bitN)<=circuitN) break;

		cout << minLevel+2 << endl;
	#endif
	

	

		
}
